#include "stm32f10x.h"

#define DMA_BLOCK_LENGTH 256

void init_USART();
void init_DMA();
void startUSARTDMATransferTx();
uint32_t calcCRC32HW(uint32_t * buf, u32 size32);
uint32_t calcCRC32SW(u8 * buf, u32 size, uint32_t startValue);
u32 revbit(u32 data);

uint8_t dataBuffer[DMA_BLOCK_LENGTH];
uint32_t crcValue;
uint8_t firstBlock = 1;
uint32_t dataLength = 0;
uint32_t * dataBufferPtr;
	
uint32_t calcCRC32HW(uint32_t * buf, u32 size32)
{
	while(size32--) CRC->DR = revbit(*(buf++));
	return revbit(CRC->DR);
}

uint32_t calcCRC32SW(u8 * buf, uint32_t size, uint32_t currentValue){
	while(size--)
	{
		currentValue ^= (u32)*buf++;
		for(uint32_t j=0; j<8; j++)
			if (currentValue & 1)
			  currentValue = (currentValue >> 1) ^ 0xEDB88320;
			else
			  currentValue >>= 1;
	}
	return ~currentValue;
};

int main()
{
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_CRC, ENABLE);
	init_USART();
	init_DMA();
	
	while(1) {
		
	}
}

void DMA1_Channel5_IRQHandler(void)
{
   DMA_ClearITPendingBit(DMA1_IT_TC5);
   if(firstBlock){
		CRC_ResetDR();
		dataLength = *((uint32_t *) dataBuffer);
		dataBufferPtr = (uint32_t *) dataBuffer;
		dataBufferPtr++;
	}
	if(dataLength <= (DMA_BLOCK_LENGTH + (firstBlock ? -4 : 0))){ //last block
		crcValue = calcCRC32HW(dataBufferPtr, dataLength / 4);
		dataBufferPtr += dataLength / 4;
		crcValue = calcCRC32SW((uint8_t *)dataBufferPtr, dataLength % 4, crcValue);
		startUSARTDMATransferTx();
		firstBlock = 1;
	} else { //not last block
		uint32_t processedDataLength = DMA_BLOCK_LENGTH / 4 + (firstBlock ? -1 : 0);
		calcCRC32HW(dataBufferPtr, processedDataLength);
		dataBufferPtr = (uint32_t *) dataBuffer;
		dataLength -= processedDataLength * 4;
		firstBlock = 0;
	}
}

u32 revbit(u32 data) 
{
	asm("rbit r0,r0"); 
	return data; 
};

void startUSARTDMATransferTx() {
  	DMA_Cmd(DMA1_Channel4, DISABLE);
	DMA1_Channel4->CMAR = (uint32_t)&crcValue;
	DMA1_Channel4->CNDTR = 4;
  	DMA_Cmd(DMA1_Channel4, ENABLE);
}

void init_USART(){
  
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_USART1, ENABLE);
	
	GPIO_InitTypeDef GPIO_InitStructure;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	USART_InitTypeDef USART_InitStructure;
	USART_InitStructure.USART_BaudRate = 115200;
	USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	USART_InitStructure.USART_StopBits = USART_StopBits_1;
	USART_InitStructure.USART_Parity = USART_Parity_No;
	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	USART_InitStructure.USART_Mode = USART_Mode_Tx | USART_Mode_Rx;
	USART_Init(USART1, &USART_InitStructure);
}

void init_DMA(){
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);
	
	DMA_InitTypeDef DMA_InitStructure;
	
	DMA_DeInit(DMA1_Channel5);
	DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)&(USART1->DR);
	DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t) dataBuffer;
	DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
	DMA_InitStructure.DMA_BufferSize = DMA_BLOCK_LENGTH;
	DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
	DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
	DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
	DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
	DMA_InitStructure.DMA_Priority = DMA_Priority_High;
	DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
	DMA_Init(DMA1_Channel5, &DMA_InitStructure);
	
	DMA_DeInit(DMA1_Channel4);
	DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)&crcValue;
	DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
	DMA_InitStructure.DMA_BufferSize = 4;
	DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
	DMA_Init(DMA1_Channel4, &DMA_InitStructure);
	
	NVIC_InitTypeDef NVIC_InitStructure;
	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_0);
	NVIC_InitStructure.NVIC_IRQChannel = DMA1_Channel5_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
	
	DMA_ITConfig(DMA1_Channel5, DMA_IT_TC, ENABLE);
	USART_DMACmd(USART1, USART_DMAReq_Rx | USART_DMAReq_Tx, ENABLE);
	DMA_Cmd(DMA1_Channel5, ENABLE);
	USART_Cmd(USART1, ENABLE);
}
